# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# stdlib imports
import json
from os.path import abspath, dirname, join

# 3rd party imports
from flask import request, redirect
from jinja2 import Environment, PackageLoader, select_autoescape

# local imports
from . import app
from .. import logic
from .. import nginx


TEMPLATE_DIR = join(abspath(dirname(__file__)), 'templates')
env = Environment(
    loader=PackageLoader('appconf.wsgi', 'templates'),
    autoescape=select_autoescape(['html', 'xml'])
)


def json_response(code, data, headers=None):
    headers = dict(headers or {})
    headers['Content-Type'] = 'application/json'

    return json.dumps(data), code, headers


@app.route('/login', methods=['get'])
def login_form():
    """ List all apps on the system. """
    template = env.get_template('login.html')
    return template.render()


@app.route('/login', methods=['post'])
def login():
    """ List all apps on the system. """
    args = request.form if request.json is None else request.json
    username = args.get('username')
    password = args.get('password')

    return json_response(200, {
        'username': username,
        'password': password
    })


@app.route('/')
def index():
    apps = logic.AppManager(app.config['CONF_DIR'])
    sites = apps.list([])

    template = env.get_template('index.html')

    fields = nginx.Site.fields()
    fields.remove('config')
    fields.remove('path')

    return template.render(
        fields=fields,
        sites=sites,
        next='/'
    )


@app.route('/apps')
def list():
    """ List all apps on the system. """
    apps = logic.AppManager(app.config['CONF_DIR'])
    sites = apps.list([])

    return json_response(200, [
        site.as_dict() for site in sites
    ])


@app.route('/add', methods=['get'])
def add_form():
    default_domain = 'novocode.net'
    default_host = '127.0.0.1'

    return env.get_template('add.html').render(**locals())


@app.route('/add', methods=['post'])
def add():
    args = request.form if request.json is None else request.json
    next = request.form.get('next')

    if not args['name']:
        return json_response(400, {'detail': 'name is required'})

    if not args['port']:
        return json_response(400, {'detail': 'port is required'})

    app_config = {
        'name': args['name'],
        'port': args['port'],
        'host_addr': args.get('host_addr', '127.0.0.1'),
        'max_body_size': args.get('max_body_size', '20M'),
        'domain': args.get('domain', 'novocode.net'),
    }

    apps = logic.AppManager(app.config['CONF_DIR'])
    apps.add(**app_config)

    if next is not None:
        return redirect(next)
    else:
        return json_response(201, app_config)


@app.route('/delete/<name>', methods=['post'])
def delete(name):
    next = request.form.get('next')
    apps = logic.AppManager(app.config['CONF_DIR'])

    try:
        apps.delete(name)

        if next is not None:
            return redirect(next)
        else:
            return json_response(201, app_config)
    except logic.AppDoesNotExist:
        return json_response(400, {
            'detail': "'{}' does not exist!".format(name)
        })


@app.route('/start/<name>', methods=['post'])
def start(name):
    next = request.form.get('next')

    try:
        apps = logic.AppManager(app.config['CONF_DIR'])
        apps.start(name)

        if next is not None:
            return redirect(next)
        else:
            return json_response(200, {
                'detail': "'{}' has  been started".format(name)
            })
    except logic.AppDoesNotExist:
        return json_response(400, {
            'detail': "'{}' does not exist!".format(name)
        })


@app.route('/stop/<name>', methods=['post'])
def stop(name):
    next = request.form.get('next')

    try:
        apps = logic.AppManager(app.config['CONF_DIR'])
        apps.stop(name)

        if next is not None:
            return redirect(next)
        else:
            return json_response(200, {
                'detail': "'{}' has  been started".format(name)
            })
    except logic.AppDoesNotExist:
        return json_response(400, {
            'detail': "'{}' does not exist!".format(name)
        })



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# local imports
from .app import db


class Site(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(80), unique=True, nullable=False)
    server_name = db.Column(db.String(500))
    status = db.Column(db.String(10))
    listen = db.Column(db.String(100), default='80')
    app_addr = db.Column(db.String(500))






# -*- coding: utf-8 -*-
""" appconf high level logic. """
from __future__ import absolute_import, unicode_literals

# stdlib imports
import os
from os.path import abspath, exists, join, relpath

# local imports
from . import nginx
from . import util


class AppDoesNotExist(RuntimeError):
    """ Raised when app with the given name does not exist. """
    pass


class AppManager(object):
    """ Encapsulates one nginx configuration location. """
    def __init__(self, conf_dir):
        self.conf_dir = conf_dir

    def list(self, filters):
        """ Get list of existing apps. """
        sites = nginx.collect_sites(self.conf_dir)
        sites = util.filter_sites(sites, filters)
        return sites

    def get(self, name):
        sites = nginx.collect_sites(self.conf_dir)
        return next((s for s in sites if s.name == name), None)

    def add(self, name, host_addr, port, domain, max_body_size):
        """ Add new app. """
        app_config = nginx.gen_app_config(
            port=port,
            name=name,
            domain=domain,
            max_body_size=max_body_size,
            host_addr=host_addr
        )

        config_path = join(self.conf_dir, 'sites-available', name + '.conf')

        with open(config_path, 'w') as fp:
            fp.write(app_config)

        return app_config

    def delete(self, name):
        filename = name + '.conf'
        conf_path = abspath(join(self.conf_dir, 'sites-available', filename))
        live_path = abspath(join(self.conf_dir, 'sites-enabled', filename))

        if not exists(conf_path):
            raise AppDoesNotExist("Site {} does not exist!".format(name))

        if exists(live_path):
            os.remove(live_path)

        os.remove(conf_path)

    def start(self, name):
        """ Start an app with the given name. """
        filename = name + '.conf'

        conf_path = abspath(join(self.conf_dir, 'sites-available', filename))
        live_path = abspath(join(self.conf_dir, 'sites-enabled', filename))
        live_dir = abspath(join(self.conf_dir, 'sites-enabled'))

        if not exists(conf_path):
            raise AppDoesNotExist("Site {} does not exist!".format(name))

        if exists(live_path):
            # Already live
            return

        cwd = os.getcwd()
        try:
            os.chdir(live_dir)
            os.symlink(relpath(conf_path, live_dir), filename)
        finally:
            os.chdir(cwd)

    def stop(self, name):
        """ Stop an app with the given name. """
        filename = name + '.conf'

        conf_path = abspath(join(self.conf_dir, 'sites-available', filename))
        live_path = abspath(join(self.conf_dir, 'sites-enabled', filename))

        if not exists(conf_path):
            raise AppDoesNotExist("Site {} does not exist!".format(name))

        if not exists(live_path):
            return

        os.remove(live_path)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# stdlib imports
from os.path import join

# 3rd party imports
from flask import send_from_directory, send_file

# local imports
from appconf import logic
from appconf import nginx
from .. import app
from .common import jinja, WEBAPP_DIR


DIST_DIR = join(WEBAPP_DIR, 'static')


@app.route('/dist/<path:path>')
def frontend_files(path):
    return send_from_directory(DIST_DIR, path)


@app.route('/dash')
def dashboard():
    apps = logic.AppManager(app.config['CONF_DIR'])
    sites = apps.list([])

    template = jinja.get_template('index.html')

    fields = nginx.Site.fields()
    fields.remove('config')
    fields.remove('path')

    return template.render(
        fields=fields,
        sites=sites,
        next='/'
    )


@app.route('/', defaults={'path': ''})
@app.route('/<path:path>')
def index(path):
    print("CATCH-ALL: {}".format(path))
    return send_file(join(DIST_DIR, 'index.html'))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# Import app as it is an interface for the whole package. Can be easily imported
# with from appconf.wsgi import app
from .app import app

# Import all routes
from .routes import *



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# stdlib imports
import json
from os.path import abspath, dirname, join, normpath

# 3rd party imports
from jinja2 import Environment, PackageLoader, select_autoescape


WEBAPP_DIR = normpath(abspath(join(dirname(__file__), '..')))
jinja = Environment(
    loader=PackageLoader('appconf.wsgi', 'templates'),
    autoescape=select_autoescape(['html', 'xml'])
)


def json_response(code, data, headers=None):
    headers = dict(headers or {})
    headers['Content-Type'] = 'application/json'

    return json.dumps(data), code, headers



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# 3rd party imports
from flask import request, redirect
# from flask_jwt import jwt_required, current_identity

# local imports
from appconf import logic
from .. import app
from .common import json_response
from appconf.wsgi.auth import jwt_required


@app.route('/api/app')
@jwt_required()
def app_list():
    """ List all apps on the system. """
    apps = logic.AppManager(app.config['CONF_DIR'])
    sites = apps.list([])

    return json_response(200, [
        site.as_dict() for site in sites
    ])


@app.route('/api/app', methods=['post'])
def app_create():
    args = request.form if request.json is None else request.json
    next = request.form.get('next')

    if not request.json['name']:
        return json_response(400, {'detail': 'name is required'})

    if not request.json['port']:
        return json_response(400, {'detail': 'port is required'})

    site = {
        'name': request.json['name'],
        'port': request.json['port'],
        'host_addr': request.json.get('host_addr', '127.0.0.1'),
        'max_body_size': request.json.get('max_body_size', '20M'),
        'domain': request.json.get('domain', 'novocode.net'),
    }

    sites = logic.AppManager(app.config['CONF_DIR'])
    sites.add(**site)

    return json_response(201, site)


@app.route('/api/app/<name>', methods=['delete'])
def app_delete(name):
    next = request.form.get('next')
    sites = logic.AppManager(app.config['CONF_DIR'])

    try:
        sites.delete(name)

        if next is not None:
            return redirect(next)
        else:
            return json_response(204, {})
    except logic.AppDoesNotExist:
        return json_response(400, {
            'detail': "'{}' does not exist!".format(name)
        })


@app.route('/api/app/<name>/config')
def app_config(name):
    """ List all apps on the system. """
    apps = logic.AppManager(app.config['CONF_DIR'])
    webapp = apps.get(name)

    if webapp is None:
        return json_response(400, {
            'detail': "'{}' does not exist!".format(name)
        })

    return webapp.config, 200, {'Content-Type': 'text/plain'}


@app.route('/api/app/<name>/start', methods=['post'])
def app_start(name):
    next = request.form.get('next')

    try:
        apps = logic.AppManager(app.config['CONF_DIR'])
        apps.start(name)

        if next is not None:
            return redirect(next)
        else:
            webapp = apps.get(name)
            return json_response(200, {
                'detail': "'{}' has  been started".format(name),
                'app': webapp.as_dict()
            })
    except logic.AppDoesNotExist:
        return json_response(400, {
            'detail': "'{}' does not exist!".format(name)
        })


@app.route('/api/app/<name>/stop', methods=['post'])
def app_stop(name):
    next = request.form.get('next')

    try:
        apps = logic.AppManager(app.config['CONF_DIR'])
        apps.stop(name)

        if next is not None:
            return redirect(next)
        else:
            webapp = apps.get(name)
            return json_response(200, {
                'detail': "'{}' has  been started".format(name),
                'app': webapp.as_dict()
            })
    except logic.AppDoesNotExist:
        return json_response(400, {
            'detail': "'{}' does not exist!".format(name)
        })



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# stdlib imports
import json
from datetime import timedelta
from os.path import dirname, join

# 3rd party imports
from flask import Flask
from flask_sqlalchemy import SQLAlchemy


app = Flask(__name__)
app.debug = True
db = SQLAlchemy()


def init_app():
    from .auth import jwt
    cfg = json.load(open(join(dirname(__file__), 'run/config.json')))

    converters ={
        'JWT_EXPIRATION_DELTA': lambda x: timedelta(seconds=x)
    }

    for name, converter in converters.items():
        if name in cfg:
            cfg[name] = converter(cfg[name])

    app.config.update(cfg)

    jwt.init_app(app)
    db.init_app(app)
    db.create_all()



# -*- coding: utf-8 -*-
""" Logging helpers """
from __future__ import absolute_import, unicode_literals

# stdlib imports
import re
import sys


class ShellColors(object):
    """ Simple utility for coloring shell output """
    COLOR_RE = re.compile(r'\^(\d{1,2})')

    def __init__(self, opcode):
        """ Create color formatter using the given opcode """
        self.opcode = opcode

    def __call__(self, msg, *args, **kw):
        """
        :param msg: Message
        :param args: Positional arguments for msg.format()
        :param kw: Keyword arguments for msg.format()
        :return: Formatted string
        """
        if len(args) or len(kw):
            msg = msg.format(*args, **kw)

        return ShellColors.COLOR_RE.sub(self.opcode, msg)


cfmt = ShellColors(opcode=r'\033[\1m')


def debug(msg, *args, **kw):
    """ Log debug message. """
    if len(args) or len(kw):
        msg = msg.format(*args, **kw)

    log_raw(cfmt('-- ^90{}^0', msg))


def info(msg, *args, **kw):
    """ Log info message. """
    if len(args) or len(kw):
        msg = msg.format(*args, **kw)

    log_raw(cfmt('-- ^32{}^0', msg))


def warning(msg, *args, **kw):
    """ Log warning message. """
    if len(args) or len(kw):
        msg = msg.format(*args, **kw)

    log_raw(cfmt('-- ^33{}^0', msg))


def err(msg, *args, **kw):
    """ Log error message. """
    if len(args) or len(kw):
        msg = msg.format(*args, **kw)

    log_raw(cfmt('-- ^31{}^0', msg))


def log_raw(msg):
    """ Raw log message.

    All other logging functions should call this one in the end.
    """
    print(msg, file=sys.stderr)



# -*- coding: utf-8 -*-
""" nginx related code. """
from __future__ import absolute_import, unicode_literals

# stdlib imports
import os
import re
from fnmatch import fnmatch
from os.path import exists, join, splitext

# 3rd party imports
import attr
import nginx as pynginx
from six import string_types

# local imports
from . import nginx
from . import util


# Site = namedtuple('Site', 'name path status listen host port hostname config')


@attr.s
class Site(object):
    """ Represents one existing nginx site. """
    name = attr.ib()
    path = attr.ib()
    status = attr.ib('disabled')
    listen = attr.ib(default=None)
    host = attr.ib(default=None)
    port = attr.ib(default=None)
    hostname = attr.ib(default=None)
    config = attr.ib(default=None)

    @classmethod
    def fields(cls):
        """ Return the list of field names on the site instance. """
        return [field.name for field in cls.__attrs_attrs__]

    def as_row(self, fields=None):
        """ Return the selected fields as a tuple. """
        fields = fields or self.fields()
        return tuple(getattr(self, name) for name in fields)

    def as_dict(self):
        """ Return the site as a python dictionary. """
        ret = attr.asdict(self)
        del ret['config']
        return ret

    @property
    def pretty_port(self):
        if self.port is None:
            return ''
        elif (isinstance(self.port, string_types) and
                self.port.startswith('unix:/')):
            return 'unix'
        else:
            return self.port

    @property
    def pretty_port(self):
        if self.port is None:
            return ''
        elif (isinstance(self.port, string_types) and
                self.port.startswith('unix:/')):
            return 'unix'
        else:
            return self.port


class SiteFilter(object):
    """ Filter for site lists. """
    RE_FILTER = re.compile(r'[\w\d_]+=.*')

    def __init__(self, field, pattern):
        self.field = field
        self.pattern = pattern

    @classmethod
    def from_string(cls, value):
        if not SiteFilter.RE_FILTER.match(value):
            raise ValueError(
                'Invalid filter. '
                'The filter should be in format <attr>=<value query>'
            )
        field, pattern = value.split('=')

        if field not in nginx.Site.fields():
            raise ValueError(
                'Invalid filter field. Must be one of: '
                '/'.join(nginx.Site.fields())
            )

        return SiteFilter(field, pattern)

    def __repr__(self):
        """ Return the filter in pretty format"""
        return '{} = {}'.format(self.field, self.pattern)

    def match(self, site):
        """ Return True if the site matches the filter. """
        if hasattr(site, self.field):
            value = getattr(site, self.field)

            if fnmatch(str(value), self.pattern):
                return True

        return False


def parse_config(path):
    """ Extract useful information from an nginx config file. """
    cfg = pynginx.load(open(path))

    upstreams = [
        parse_upstream(x)
        for x in cfg.children if isinstance(x, pynginx.Upstream)
    ]
    servers = [
        parse_server(x)
        for x in cfg.children if isinstance(x, pynginx.Server)
    ]

    app_server = upstreams[0]['server']
    host, port = app_server.split(':', 1)
    domain = servers[0]['domain']
    hostname = servers[0]['hostname']
    listen = '/'.join(s['listen'] for s in servers)

    assert len(set(s['hostname'] for s in servers)) == 1

    return {
        'listen': listen,
        'host': host,
        'port': port,
        'domain': domain,
        'hostname': hostname
    }


def parse_upstream(upstream):
    """ Parse upstream entry in nginx config. """
    name = upstream.as_list[1]
    server = upstream.as_list[2][0][1]

    if server.startswith('unix:/'):
        host = 'localhost'
        port = server.split(maxsplit=1)[0]
    else:
        parts = server.rsplit(':', maxsplit=1)
        host, port = parts
        if port.isdigit():
            port = int(port)

    return {
        'name': name,
        'server': server,
        'host': host,
        'port': port
    }


def parse_server(server):
    """ Parse server entry in nginx config. """
    listen = _get_key(server, 'listen')
    hostname = _get_key(server, 'server_name')
    name, domain = hostname.split('.', maxsplit=1)

    return {
        'hostname': hostname,
        'name': name,
        'domain': domain,
        'listen': listen,
    }


def _get_key(server, name):
    """ Return the value for the key with the given name. """
    return next((k.value for k in server.children if k.name == name), None)


def collect_sites(conf_dir='/etc/nginx'):
    """ Collect all sites registered with nginx. """
    sites = []
    available_path = join(conf_dir, 'sites-available')

    if exists(available_path):
        for filename in os.listdir(available_path):
            name, _ = splitext(filename)
            path = join(available_path, filename)

            live_conf_path = join(conf_dir, 'sites-enabled', filename)
            if exists(live_conf_path):
                status = 'enabled'
            else:
                status = 'disabled'

            try:
                config = open(path).read()
                parsed = parse_config(path)
                listen = parsed['listen']
                host = parsed['host']
                port = parsed['port']
                hostname = parsed['hostname']

            except IOError:
                status = 'broken'
                config = 'File not found'
                listen = host = port = hostname = None

            sites.append(Site(
                name=name,
                path=path,
                status=status,
                listen=listen,
                host=host,
                port=port,
                hostname=hostname,
                config=config,
            ))

    return sites


def gen_app_config(
        port, name, domain,
        max_body_size='200M',
        host_addr='127.0.0.1'
):
    """ Generate new nginx configuration for an app. """
    return util.remove_indent("""
        upstream {name} {{
            server {host_addr}:{port};
            keepalive 60;
        }}


        server {{
            listen                  80;
            server_name             {name}.{domain};
            client_max_body_size    {max_body_size};

            access_log /var/log/nginx/{name}/access.log;
            error_log /var/log/nginx/{name}/error.log;

            location / {{
                proxy_pass         http://{name};
                proxy_set_header   Host             $host;
                proxy_set_header   X-Real-IP        $remote_addr;
                proxy_set_header   X-Forwarded-For  $proxy_add_x_forwarded_for;
                proxy_set_header   X-Forwarded-Host $server_name;
            }}
        }}
    """.format(
        port=port,
        name=name,
        domain=domain,
        max_body_size=max_body_size,
        host_addr=host_addr,
    )).strip() + os.linesep     # Removes the start/end empty lines.



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# 3rd party imports
from jwt import InvalidTokenError
from flask import request, jsonify
from flask_jwt import JWT, JWTError, jwt_required, current_identity

# local imports
from appconf.wsgi import app
from appconf.wsgi.users import User

def user_from_payload(payload):
    return User.query.get(payload['identity'])


jwt = JWT(
    authentication_handler=lambda name, pw: User.authenticate(name, pw),
    identity_handler=user_from_payload
)


def identity_from_token(token):
    try:
        return jwt.jwt_decode_callback(token)['identity']
    except InvalidTokenError:
        return None


@jwt.auth_response_handler
def send_token(access_token, identity):
    return jsonify({'jwt_token': access_token.decode('utf-8')})


@app.after_request
def update_auth(response):
    # Only return refreshed token for API calls.
    if response.content_type == 'application/json':
        auth = request.headers.get('Authorization')
        if auth is not None:
            user_id = identity_from_token(auth.rsplit(maxsplit=1)[-1])

            if user_id is not None:
                token = jwt.jwt_encode_callback(User(id=user_id))
                response.headers['X-JWT-Token'] = token

    return response



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# Import all routes so they can be easily imported with from routes import *
from .apps import *

# Static should be imported last as it contains a catch-all route
from .static import *



# -*- coding: utf-8 -*-
# pylint: disable=redefined-builtin
""" Command line interface for appconf. """
from __future__ import absolute_import, unicode_literals

# stdlib imports
import json
from getpass import getpass
from os.path import abspath

# 3rd party imports
import click
import click_completion

# local imports
from . import log
from . import nginx
from . import util
from . import logic


click_completion.init()


@click.group()
@click.option(
    '-c', '--conf-dir',
    type=click.Path(file_okay=False),
    default='/etc/nginx'
)
@click.pass_context
def cli(ctx, conf_dir):
    """ Main command group. """
    ctx.obj = {'conf_dir': abspath(conf_dir)}


@cli.command()
@click.argument('name')
@click.option(
    '-h', '--host-addr',
    type=str,
    default='127.0.0.1',
    help=('IP address for the host that actually has the service running. '
          'If appconf is used just to manage local services, then this will '
          'always be 127.0.0.1')
)
@click.option(
    '-p', '--port',
    type=int,
    required=True,
    help=('The port the service is running on. This combined with --host-addr '
          'forms the full proxy address this server entry will use.')
)
@click.option(
    '-d', '--domain',
    type=str,
    default=None,
    help=('The domain this service is running on. Combined with the service'
          'name this forms a full server hostname.')
)
@click.option(
    '-b', '--max-body-size',
    type=str,
    default='200M',
    help=('Maximum request body size allowed.')
)
@click.option(
    '--dry-run',
    is_flag=True,
    help=('If this flag is set, no changes will be made to the nginx '
          'configuration. It will just generate and print the app config. ')
)
@click.pass_context
def create(ctx, name, host_addr, port, domain, max_body_size, dry_run):
    """ Create new app configuration. """
    log.info("Generating app configuration for ^35{}".format(name))

    args = {
        'name': name,
        'port': port,
        'host_addr': host_addr,
        'domain': domain or 'novocode.net',
        'max_body_size': max_body_size
    }

    if dry_run:
        app_config = nginx.gen_app_config(**args)
    else:
        apps = logic.AppManager(ctx.obj['conf_dir'])
        app_config = apps.add(**args)

    print(app_config)


@cli.command()
@click.option(
    '-f', '--filter',
    type=nginx.SiteFilter.from_string,
    multiple=True
)
@click.option(
    '--format',
    type=str,
    default='pretty',
    help="Output format (pretty/json)"
)
@click.pass_context
def list(ctx, filter, format):
    """ List all apps on the system. """
    apps = logic.AppManager(ctx.obj['conf_dir'])
    sites = apps.list(filter)

    if format == 'pretty':
        util.print_sites_pretty(sites)
    elif format == 'json':
        util.print_sites_json(sites)
    else:
        log.err("Invalid format {}. Supported formats are pretty/json")


@cli.command()
@click.argument('name')
@click.pass_context
def start(ctx, name):
    """ Start an app. This will require manual nginx restart afterwards. """
    log.info("Starting ^1{}".format(name))

    try:
        apps = logic.AppManager(ctx.obj['conf_dir'])
        apps.start(name)
    except logic.AppDoesNotExist:
        log.err("Site {} does not exist!".format(name))


@cli.command()
@click.argument('name')
@click.pass_context
def stop(ctx, name):
    """ Stop an app. This will require manual nginx restart afterwards. """
    log.info("Stopping ^1{}".format(name))

    try:
        apps = logic.AppManager(ctx.obj['conf_dir'])
        apps.stop(name)
    except logic.AppDoesNotExist:
        log.err("Site {} does not exist!".format(name))


@cli.command()
@click.argument('name')
@click.pass_context
def delete(ctx, name):
    """ Delete an app. This will require manual nginx restart afterwards. """
    log.info("Deleting ^1{}".format(name))

    try:
        apps = logic.AppManager(ctx.obj['conf_dir'])
        apps.delete(name)
    except logic.AppDoesNotExist:
        log.err("Site {} does not exist!".format(name))


@cli.command()
@click.option('-p', '--port', type=int, default=14001)
@click.option('--ssl', type=str, default='adhoc')
@click.option('--no-reload', is_flag=True)
@click.pass_context
def webui(ctx, ssl, no_reload, port=None):
    from appconf.wsgi.app import app, init_app

    opts = {
        'host': '0.0.0.0',
        'port': port
    }

    if ssl != 'off':
        opts['ssl_context'] = ssl

    if no_reload:
        opts['use_reloader'] = False

    app.config['CONF_DIR'] = ctx.obj['conf_dir']
    app.app_context().push()
    init_app()

    app.run(**opts)


@cli.command('create-user')
@click.argument('name')
@click.argument('email')
def create_user(name, email):
    while True:
        password1 = getpass("Password: ")
        password2 = getpass("Repeat: ")

        if password1 != password2:
            print("Error: Password do not match!")
            print("Try again.")
        else:
            break

    from appconf.wsgi.app import app, init_app
    from appconf.wsgi.users import User

    app.app_context().push()
    init_app()

    user = User.create(name, password1, email)
    print("User {} created!".format(user.name))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# 3rd party imports
from passlib.hash import pbkdf2_sha512 as passlib

# local imports
from .app import db


class User(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(80), unique=True, nullable=False)
    pw_hash = db.Column(db.String(80), nullable=False)
    email = db.Column(db.String(120), unique=True)

    def set_password(self, password):
        self.pw_hash = passlib.hash(password)

    @classmethod
    def create(cls, username, password, email):
        user = cls(name=username, email=email)

        user.set_password(password)

        db.session.add(user)
        db.session.commit()

        return user

    @classmethod
    def authenticate(cls, username, password):
        user = cls.query.filter(User.name == username).scalar()

        if user is None:
            return

        return user if passlib.verify(password, user.pw_hash) else None



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="appconf",
    url="https://github.com/novopl/appconf",
    version=read('VERSION').strip(),
    author="Mateusz 'novo' Klos",
    author_email="novopl@gmail.com",
    license="MIT",
    description="Helps generate & manage configs for webapps",
    long_description=read('README.rst'),
    package_dir={'': 'src'},
    packages=find_packages('src'),
    install_requires=[
        l.strip() for l in read('requirements.txt').split() if '==' in l
    ],
    package_data={
        'appconf.wsgi': ['run/config.json', 'static/*']
    },
    entry_points={
        'console_scripts': [
            'appconf = appconf.cli:cli',
        ]
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Topic :: Utilities",
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
    ],
)



# -*- coding: utf-8 -*-
""" Various utilities used across the code base."""
from __future__ import absolute_import, unicode_literals

# stdlib imports
import os
import json

# 3rd party imports
from tabulate import tabulate

# local imports
from . import log
from . import nginx


def remove_indent(text):
    """ Remove common indent from the text. """
    lines = text.split(os.linesep)

    min_indent = -1
    for line in (l for l in lines if l.strip()):
        without_indent = line.lstrip()
        indent = len(line) - len(without_indent)

        if min_indent == -1 or indent < min_indent:
            min_indent = indent

    return '\n'.join(l[min_indent:] for l in lines)


def filter_sites(sites, filters):
    """ Filter site list using a set of given filters. """
    if not filters:
        return sites

    results = []

    for site in sites:
        if next((True for f in filters if f.match(site)), False):
            results.append(site)

    return results


def print_sites_json(sites, indent=2):
    """ Print sites as JSON. """
    print(json.dumps([site.as_dict() for site in sites], indent=indent))


def print_sites_pretty(sites):
    """ Pretty print site list as a colored table. """
    fields = nginx.Site.fields()
    fields.remove('config')

    print(tabulate(
        [s.as_row(fields) for s in colored_sites(sites)],
        headers=fields
    ))


def colored_sites(sites):
    """ Colour selected fields in a Site instance"""
    for s in sites:
        colored = nginx.Site(**s.as_dict())
        colored.name = log.cfmt('^1{}^0', s.name)
        colored.status = _format_status(s.status)

        yield colored


def _format_status(status):
    """ Add color to the app status. """
    if status == 'enabled':
        color = '^32'
    elif status == 'broken':
        color = '^31'
    else:
        color = '^90'

    return log.cfmt(color + status + '^0')
